/* CALENDAR.C  Basic 'to-do' program */

#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <stdio.h>
#include <dos.h>
#include <ctype.h>
#include <dir.h>

int     CALSZ;                /* 366 if leap year, else 365 */
#define NUMITEMS  10
#define ITEMSZ    50
#define LOCAL    40           /* size of working array */

struct LINE {
         int date;               /* day_num for current year */
         int top;                /* 1 if date line, else 0 */
         int num;                /* number of entries for this date */
         char string[ITEMSZ+5];  /* either date_string or item */
       } line[LOCAL*(NUMITEMS + 1)];

#define ESC       27
#define F1       -59
#define CR        13
#define UP       -72
#define DN       -80
#define PgUp     -73
#define PgDn     -81
#define RT       -77
#define LF       -75
#define BK        8
#define Del      -83
#define Ins      -82
#define Home     -71
#define End      -79
#define Ctrl_END -141
#define Ctrl_UP  -141
#define Ctrl_DN  -145
#define Ctrl_Y    25
#define Ctrl_L    12
#define Alt_J    -36
#define Alt_E    -18
#define Alt_N    -49

/* function declarations */
char *tail(int c,int r, int c1);
char *top_string(char *title1, char *title2);
void
  calscrn(int year),
  delete(int c,int r,int c1),
  delete_line(int),
  down_one(),
  down_one_page(),
  display_lines(int startdate),
  enter_text(int scol, int srow, int length, int num_rows),
  helpmsgs(),
  get_entry(int date),
  insert_space(int c, int r, int c1),
  month_day(int year, int date, int *month, int *day, char *date_str),
  normal_exit(char *error_msg),
  store_line(int r, int r0, int c0),
  rect(int left, int top, int right, int bottom, int attr, int border),
  setcursor(int),                           /* 0=normal, 1=large, 2=hide */
  up_one(),
  up_one_page();
int
  all_blanks(char *s),
  carryover(int date, char *item),          /* add item to next year's cal */
  chk_date(int mm, int dd, int yy),
  day_of_year(int month, int day, int year),
  Getint(int n, char *s, int vi[]),
  getkey(void),
  gstr(char *s, int col, int row, int len), /* improved gets() */
  remainder(int x, int y),                  /* correction of x % y */
  new_date(int *m, int *d, int *y),
  read_char(int col, int row),
  read_date(char *s,int *,int *,int *),  /*  set month, day, year from s[] */
  size(int date),
  day_of_week(int month, int day, int year);

/* box characters */
int tlc[3]={0,218,201};
int trc[3]={0,191,187};
int blc[3]={0,192,200};
int brc[3]={0,217,188};  /* single,double corners */
int hl[3]={0,196,205};
int vl[3]={0,179,186};   /* single,double horizontal, vertical lines */

int day_tab[2][13] = {
     {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
     {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
char *month_name[] =
  {"","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
char *day_name[] = {" Sat "," Sun "," Mon "," Tue "," Wed "," Thu "," Fri "};


FILE *fp, *gp;
int startdate,mindate,maxdate;
int index;
int month,day,year,leap;
int norm,high;
int mono;
int old_attr;
int row,r,c;
int r0=1,r1=21;
int c0=3,c1=63;
int dx, dy;
char *basename = "caldata.";
char datafile[20];
char nextdata[20];
int num_lines, maxindex;
int indx[24];   /* display_lines puts line[indx[r]].string on row r */

struct LINE line[LOCAL*(NUMITEMS+1)];
char string[NUMITEMS][ITEMSZ+1];

long entered[367];                           /* offsets of dates in datafile */
long entered1[367];

/*---------------------------------------------------------------------------*/
main(int argc, char *argv[])
{
  char s[80];
  int vi[3];
  int i,j,k,n;
  char year_str[5];
  struct text_info oldtext;
  int oldyear = 0;
  int m,d,y,date;
  long zero = 0L;                  /* for use in initializing caldata.yy */
  char *dir = "c:\\cal";
  struct ffblk ffblk;

  maxdate = 1;
  gettextinfo(&oldtext);            /* original text settings */
  old_attr = oldtext.normattr;
  if (oldtext.currmode == '\7') {
    mono = 1;
    norm = 7;
    high = 16*7;
  }
  else {
    mono = 0;
    norm = 16*3+8;
    high = 16*7+8;
  }

  if (argc > 1) {        /* combine all the arguments into a single string */
    strcpy(s,argv[1]);
    for (i=2; i<argc; i++) {
      strcat(s," ");
      strcat(s,argv[i]);
    }
    if (!read_date(s,&month,&day,&year) || year == 0) {
      printf("\n\t Usage: CAL date  (for specified date)\n");
      printf("\t    or: CAL       (for the current date)\n");
      printf("\t The 'date' may be given either in ordinary english\n");
      printf("\t (e.g. 'July 4, 1992' or numerically (mm/dd/yy or\n");
      printf("\t mm-dd-yy or m,d,y or ...\n");
      printf("\t (three values are required in any case.)\n");
      exit(0);
    }
  }
  else {                  /* get system date if none on command line */
    _AH = 0x2A;
    geninterrupt(0x21);
    month = _DH;
    day = _DL;
    year = _CX;
  }

  if (year < 1000)
    year += 1900;      /* most routines which use 'year' need 4-digit value */
  if (!chk_date(month,day,year)) {    /* check for valid 20th century date */
    printf("Invalid date");
    exit(1);
  }
  if (findfirst(dir,&ffblk,FA_DIREC) != 0)
    dir[0] = '\0';

a:                      /* come here if new month,day,year entered */
  leap = year%4 == 0 && year%100 != 0 || year%400 == 0;
  if (leap) CALSZ = 366; else CALSZ = 365;
  startdate = day_of_year(month,day,year);
  if (year != oldyear) {
    oldyear = year;    /* save current year in case changed via new_date() */
    itoa(year%1900, year_str, 10);  /* here a 2-digit year is needed */
    if (dir[0]) {             /* directory c:\cal exists */
      strcpy(datafile,dir);
      strcat(datafile,"\\");
      strcpy(nextdata,dir);
      strcat(nextdata,"\\");
    }
    else
      datafile[0] = nextdata[0] = '\0';
    strcat(datafile,basename);
    strcat(datafile,year_str);
    itoa(year%1900+1, year_str, 10);  /* next year */
    strcat(nextdata,basename);
    strcat(nextdata,year_str);
    if (gp != NULL) fclose(gp);  /* just in case it was opened and not closed */
    /* open datafile and read array 'entered[]' */
    gp = fopen(datafile,"rb+");   /* contains all the 'to do' lines */
    if (gp == NULL) {             /* if file doesn't exist, create it */
      gp = fopen(datafile,"wb");
      for (i=0; i<CALSZ; i++)
        fwrite(&zero, sizeof(long), 1, gp);
      fclose(gp);
      gp = fopen(datafile,"rb+");
    }
    if (gp == NULL)
      normal_exit("Unable to find or create datafile");
    if (fread(entered, sizeof(long), CALSZ, gp) != CALSZ) {
      normal_exit("Error reading 'caldata.yy'");
      exit(1);
    }
    rewind(gp);
  }
  calscrn(year);
  display_lines(startdate);

  c=c0; r=r0;

  while(1) {
    if (r < r0) r = r0;
    if (r > r1) r = r1;
    i = indx[r];
    gotoxy(c,r);
    switch(getkey()) {
      case ESC:
        normal_exit("");

      case F1:
        helpmsgs();
        break;

      case UP:
        if (r > r0)
          r--;
        else
          up_one();
        break;

      case Ctrl_UP:
        if (r > r0)
          r = r0;
        break;

      case DN:
        if (r < r1)
          r++;
        else
          down_one();
        break;

      case Ctrl_DN:
        if (r < r1)
          r = r1;
        break;

      case PgUp:
        up_one_page();
        break;

      case PgDn:
        down_one_page();
        break;

      case Alt_J:           /* enter new startdate for lines display */
        if (new_date(&month, &day, &year)) {
          goto a;
        }
        break;

      case Alt_E:           /* enter date to edit */
        if (new_date(&month, &day, &year)) {
          startdate = day_of_year(month, day, year);
          get_entry(startdate);
          display_lines(startdate);
        }
        break;

      case CR:               /* edit date at cursor */
        get_entry(startdate = line[indx[r]].date);
        display_lines(startdate);
        break;
      }
    }
}

/*---------------------------------------------------------------------------*/
void
normal_exit(char *error_msg)
{
  int c;

  fcloseall();
  window(1,1,80,25);
  textattr(old_attr);
  clrscr();
  setcursor(0);
  if (error_msg[0]) {
    gotoxy(10,3);
    cprintf("%s\n",error_msg);
  }
  exit(0);
}

/* ---------------------------------------------------------------------- */
void
helpmsgs()
{
  int i;
  int L=5,T=2,R=74,B=21;
  char buf[3000];
  char *msg[] = {
    "                      CAL  HELP",
    "Change starting date:  PgUp/PgDn   (up/down one page)",
    "                       UP one date (when cursor on top line)",
    "                       DN one date (when cursor on bottom line)",
    "            (Ctrl-UP/Ctrl-DN move cursor to top/bottom of window.)",
    "                       Alt-J       (jump to date entered)",
    "",
    "Enter or change items: Move cursor to desired date and press Enter",
    "                       or use Alt-E to enter date to edit.",
    "",
    "Editing keys:  Arrow keys, Backspace, Ins, Del, keyboard as usual.",
    "               Home     moves to the start of the line",
    "               End      moves to the end of the line",
    "               Ctrl-End deletes from cursor to end of line",
    "               Ctrl-Y   deletes the entire line",
    "               Ctrl-L   restores the original line",
    "               Enter    moves to the start of the next line",
    "               Esc      exits the entry mode"};

    window(1,1,80,25);
    gettext(L,T,R,B,buf);
    rect(L,T,R,B,16*3+14,2);
    textattr(norm);
    window(L+1,T+1,R-1,B-1);
    clrscr();
    for (i=0; i<18; i++) {
      gotoxy(2,i+1);
      cputs(msg[i]);
    }
    while (getkey() != ESC)
      ;
    puttext(L,T,R,B,buf);
    window(9,3,71,23);
}

read_date(char *s, int *m, int *d, int *y)
{
  char *p=s;
  int cnt,vi[3] = {0,0,0};
  int month;

  strupr(s);
  if (strstr(s,"JAN"))  month = 1;
  else if (strstr(s,"FEB"))  month = 2;
  else if (strstr(s,"MAR"))  month = 3;
  else if (strstr(s,"APR"))  month = 4;
  else if (strstr(s,"MAY"))  month = 5;
  else if (strstr(s,"JUN"))  month = 6;
  else if (strstr(s,"JUL"))  month = 7;
  else if (strstr(s,"AUG"))  month = 8;
  else if (strstr(s,"SEP"))  month = 9;
  else if (strstr(s,"OCT"))  month = 10;
  else if (strstr(s,"NOV"))  month = 11;
  else if (strstr(s,"DEC"))  month = 12;
  else month = 0;
  while(*p && isspace(*p)) p++;
  if (!*p)
    return 0;
  cnt = Getint(3,s,vi)+(month != 0);
  if (cnt == 0)
    return 0;
  if (cnt < 2)
    return 0;
  if (month == 0) {
    *m = vi[0];
    *d = vi[1];
    if (cnt == 3)
      *y = vi[2];
    else
      *y = 0;
  }
  else {
    *d = vi[0];
    *m = month;
    if (cnt == 3)
      *y = vi[1];
    else
      *y = 0;
  }
  return 1;
}
void
up_one()
{
  if (startdate == 1)
    return;
  display_lines(--startdate);
}
void
down_one()
{
  if (startdate == CALSZ)
    return;
  display_lines(++startdate);
}
void
down_one_page()
{
  if (startdate == maxdate)
    return;

  display_lines(startdate = maxdate);
}
void
up_one_page()
{
  int k = 0;
  int n = 19;

  if (startdate == 1)
    return;

  startdate--;
  while (1)
  {
    k += size(startdate);
    if (startdate == 1)
      break;
    if (k + size(--startdate) > n)
      break;
  }
  display_lines(startdate);
}

size(int date)
{
  int j;

  if (!entered[date])
    return 1;

  fseek(gp,entered[date],SEEK_SET);
  fread(string,sizeof(char),sizeof(string),gp);
  j=0;
  do j++; while (j<NUMITEMS && string[j][0]);
  return j+2;
}

/*---------------------------------------------------------------------------*/
fill_lines(int date)
{
  int i,i0,j,k,l,t,n,num;
  char date_str[9];
  char weekday[5];
  char top_str[ITEMSZ+5];
  static char bottom_line[ITEMSZ+5];
  static char left_margin[] = {179, ' ','\0'};
  static char right_margin[] = {' ', 179,'\0'};
  char s[ITEMSZ+5];
  int wk,month,day;

  bottom_line[0] = blc[1];
  for (i=1; i<ITEMSZ+3; i++) bottom_line[i] = hl[1];
  bottom_line[ITEMSZ+3] = brc[1];
  bottom_line[ITEMSZ+4] = '\0';

  for (i=0; i<num_lines; i++) {
    line[i].top = 0;
    line[i].date = 0;
    line[i].num = 0;
    line[i].string[0] = 0;
  }
  date--; i = -1;
  while (i < 21 && date < CALSZ) {
    date++; i++;
    line[i].date = date;
    line[i].top = 1;
    line[i].num = 0;       /* will be changed if items have been entered */
    t = 0;       /* t will count items; final value will be line[i].num */
    month_day(year, date, &month, &day, date_str);
    wk = day_of_week(month, day, year);
    strcpy(weekday,day_name[wk]);
    strcpy(line[i].string, top_string(date_str,weekday));
    if(!entered[date]) {
      l = strlen(line[i].string) - 1;
      line[i].string[0] = 196;
      line[i].string[l-1] = 196;
      line[i].string[l] = 0;
    }
    else {
      fseek(gp,entered[date],SEEK_SET);
      n = fread(string, sizeof(char), sizeof(string),gp);
      if (n != sizeof(string))
        normal_exit("Error reading caldata.yy");
      for (j=0; j<NUMITEMS; j++) {
        if (string[j][0] == '\0')
          continue;
        if (all_blanks(string[j]))
          continue;
        t++;
        strcpy(line[i+t].string, left_margin);
        strcat(line[i+t].string, string[j]);
        strcat(line[i+t].string,right_margin);
        line[i+t].date = date;
        line[i+t].top = 0;
        line[i+t].num = 0;
      }
      line[i].num = t;
      t++;
      line[i+t].date = date;
      line[i+t].top = 0;
      line[i+t].num = 0;
      strcpy(line[i+t].string, bottom_line);
      i = i+t;
    }
  }
  rewind(gp);
  return i;
}

all_blanks(char *s)
{
  int i;

  for (i=0; i<strlen(s); i++) {
    if (s[i] != ' ')
      return 0;
    return 1;
  }
}

/*---------------------------------------------------------------------------*/
new_date(int *m, int *d, int *y)
{
  char buf[400];
  char s[20];
  int vi[3];
  int mm,dd,yy,cnt;
  int no_date = 0;

  window(1,1,80,25);
  gettext(21,4,60,8,buf);      /* screen coordinates */
  rect(21,4,60,8,16*6+14,2);
  window(22,5,59,7);
  textattr(16*6+15);
  clrscr();
a:
  gotoxy(6,2);
  clreol();
  cprintf("Enter date: ");
  getstr(s,11);
  if (!s[0]) {
    no_date = 1;
    goto b;
  }
  if (!read_date(s,&mm,&dd,&yy)) {
    gotoxy(6,2);
    cprintf("Invalid date; Please reenter");
    getch();
    goto a;
  }

  if (yy == 0) yy = year;
  if (yy < 1000) yy+= 1900;
  if (!chk_date(mm,dd,yy)) {
    gotoxy(6,2);
    cprintf("Invalid date; Please reenter");
    getch();
    goto a;
  }
  *m = mm; *d = dd; *y = yy;
b:
  window(1,1,80,25);
  puttext(21,4,60,8,buf);
  window(9,3,71,23);
  textattr(norm);
  return 1-no_date;
}

/*---------------------------------------------------------------------------*/
day_of_week(int month, int day, int year)
{
  int J,K,m=month,q=day,h;
  int Gregorian;

  if (m < 3) {
    m += 12;
    year--;
  }
  J = year/100;
  K = year % 100;
  if (J < 15 || (J == 15 && (month < 10) || (month == 10 && day < 15)))
    Gregorian = 0;
  else
    Gregorian = 1;

  h = (q + ((m+1)*26)/10 +K + K/4);
  if (Gregorian)
    h += J/4 -2*J;
  else
    h += 5 - J;

  h = remainder(h,7);
  return h;
}

/* -------------------- */
remainder(int x, int y)
{
  if (y <= 0)
    return -1;

  while (x >= y) x -= y;
  while (x < 0) x += y;
  return x;
}


/* ------------------*/
day_of_year(int month, int day, int year)
{
  int i;

  leap = year%4 == 0 && year%100 != 0 || year%400 == 0;
  for (i=1; i<month; i++)
    day += day_tab[leap][i];
  return day;
}

/* ------------------*/
void
month_day(int year, int day_num, int *month, int *day, char *s)
{
  int i,leap;

  leap = year%4 == 0 && year%100 != 0 || year%400 == 0;
  for (i=1; day_num > day_tab[leap][i]; i++)
    day_num -= day_tab[leap][i];
  sprintf(s," %s %-2d",month_name[i],day_num);
  *month = i;
  *day = day_num;
}

/* ------------------*/
/* check validity of mm/dd/yy as 20th century data */
chk_date(int mm, int dd, int yy) 
{
  int i,leap;

  if (mm < 1 || mm > 12 || dd < 1 || dd > 31 || yy < 1900 || yy > 1999)
    return 0;
  leap = yy%4 == 0 && yy%100 != 0 || yy%400 == 0;
  if (dd > day_tab[leap][mm])
    return 0;
  return 1;
}

/*---------------------------------------------------------------------------*/
void
calscrn(int year)
{
  char *command_lst =
  "   F1:Help  Alt_J:New Date  Alt_E:Date to Edit  CR:Edit Here Esc:Exit  ";
  int r;

  window(1,1,80,25);
  textattr(7);
  clrscr();
  rect(8,2,72,24,7,2);
  window(8,1,72,1);
  textattr(high);
  clrscr();
  gotoxy(30,1);
  cprintf("%d",year);
  window(1,25,80,25);
  clrscr();
  gotoxy(2,1);
  cprintf(command_lst);
  window(9,3,71,23);
  textattr(norm);
  clrscr();
}

/*---------------------------------------------------------------------------*/
void
display_lines(int startdate)
{
  int i,i0,j,row;

  textattr(norm);
  clrscr();
  row = 1;

  num_lines = fill_lines(startdate);
  i = 0;
  while (i<21 && i<=num_lines) {
    if (line[i].top == 1 && line[i].num == 0)
      textattr(norm);
    else
      textattr(high);
    indx[row] = i;
    gotoxy(6,row++);
    cputs(line[i++].string);
    maxdate = line[i-1].date;
  }
}

/* -------------------------- */
char *top_string(char *title1, char *title2)
{
  char s[80];
  static char left_top[] =
  {218,196,196,196,196,196,196,196,196,196,196,196,196,196,196,196,196,
  196,196,196,196,196,196,0};
  static char filler[] = {196,196,196,196,196,196,196,196,0};
  static char right_top[] =
  {196,196,196,196, 196,196,196,196,196,196,191,0};

  strcpy(s,left_top);
  strcat(s,title1);
  strcat(s,filler);
  strcat(s,title2);
  strcat(s,right_top);
  return s;
}

/*---------------------------------------------------------------------------*/
read_char(int col, int row)       /* return the character at (col,row) */
{                                 /* col,row absolute screen coordinates */
  _AH = 2;
  _BH = 0;
  _DH = row-1;
  _DL = col-1;
  geninterrupt(0x10);
  _AH = 8;
  _BH = 0;
  geninterrupt(0x10);
  return _AL;
}

/* ---------------------------------------------------------------------- */
void
delete(int c,int r,int c1)
{
  char s[80];

  gotoxy(c,r);
  if (c == c1)
    putch(' ');
  else {
    strcpy(s,tail(c+1,r,c1));
    strcat(s," ");
    gotoxy(c,r);
    cputs(s);
  }
}

/* ---------------------------------------------------------------------- */
void
insert_space(int c, int r, int c1)
{
  char s[80];

  strcpy(s,tail(c,r,c1));
  gotoxy(c,r);
  if (c == c1)
    putch(' ');
  else {
    s[strlen(s)-1] = '\0';
    cputs(" ");
    cputs(s);
    gotoxy(c,r);
  }
}

/* ---------------------------------------------------------------------- */
char *tail(int c,int r, int c1)
{
  int i,j;
  char s[80];

  for (i=c,j=0; i<c1; i++,j++)
    s[j] = read_char(i+dx,r+dy);
  s[j] = '\0';
  return s;
}

/* ---------------------------------------------------------------------- */
void
delete_line(int i)
{
  string[i][0] = '\0';
  clreol();
}

/* ---------------------------------------------------------------------- */
                 /* basic input loop feeder */
getkey(void)
{
 int k;

 k = getch();
 if (k == 0) k = -getch();
 return k;
}

/*---------------------------------------------------------------------------*/
/* rectangle drawing routine for borders */
void
rect(int left, int top, int right, int bottom, int attr, int border)
{
  int i;
  int width, height;

  width = right-left;
  height = bottom-top;

  textattr(attr);
  gotoxy(left,top);
  putch(tlc[border]);
  for (i=1; i<width; i++)
    putch(hl[border]);
  putch(trc[border]);
  for (i=0; i<height-1; i++) {
    gotoxy(left,top+1+i);
    putch(vl[border]);
    gotoxy(left+width,top+1+i);
    putch(vl[border]);
  }
  gotoxy(left,top+height);
  putch(blc[border]);
  for (i=1; i<width; i++)
    putch(hl[border]);
  putch(brc[border]);
}

int left = 15, top = 5, right = 65, bottom = 14;
int dx,dy;

void
get_entry(int date)
{
  char weekday[10];
  int wk;
  char title[10];
  char buffer[4000];
  int i,j,month,day,nonempty=0;
  struct text_info oldtext;

  gettextinfo(&oldtext);
  window(1,1,80,25);
  month_day(year, date, &month, &day, title);
  wk = day_of_week(month,day,year);
  strcpy(weekday,day_name[wk]);
  dx = left-1, dy = top-1;
  gettext(left-1,top-1,right+1,bottom+1,buffer);
  rect(left-1,top-1,right+1,bottom+1,16*6+14,2);
  gotoxy(40-strlen(title)/2,top-1);
  cputs(title);
  gotoxy(52,top-1);
  cputs(weekday);
  textattr(16*6+15);
  window(left,top,right,bottom);
  clrscr();
    if (entered[date]) {
      fseek(gp,entered[date],SEEK_SET);
      fread(string, sizeof(char), sizeof(string),gp);
    }
    else
      for (i=0; i< NUMITEMS; i++) string[i][0] = '\0';
  for (i=0; i<NUMITEMS; i++)
    if (string[i][0]) {
      gotoxy(2,i+1);
      cputs(string[i]);
    }
  enter_text(2,1,ITEMSZ-1,10);
  puttext(left-1,top-1,right+1,bottom+1,buffer);
  window(oldtext.winleft,oldtext.wintop,oldtext.winright,oldtext.winbottom);
  textattr(norm);
  nonempty = 0;
  for (i=0; i<NUMITEMS; i++) {
    if (string[i][0] != '\0' && !all_blanks(string[i]))
      nonempty = 1;
    if (string[i][0] == '+') {
      carryover(date, string[i]);
      string[i][0] = '*';
    }
  }
  if (nonempty) {            /* at least one string entered */
    if (entered[date])
      fseek(gp,entered[date],SEEK_SET);
    else {
      fseek(gp,0L,SEEK_END);
      entered[date] = ftell(gp);
    }
    i = fwrite(string, sizeof(char), sizeof(string), gp);
    if (i != sizeof(string)) {
      printf("Error writing to caldata.yy\n");
      exit(1);
    }
  }
  else                  /* no strings left */
    entered[date] = 0L;
  fseek(gp, 0L, SEEK_SET);
  fwrite(entered, sizeof(long), CALSZ, gp);  /* store 'entered[]' array */
}

void
enter_text(int scol, int srow, int length, int num_rows)
{
  int c0 = scol, r0 = srow, r1 = r0+num_rows-1;
  int c,r,k;

  c = c0, r = r0;
  while(1) {
    switch(gstr(string[indx[r]],c,r,length)) {

      case ESC:
        store_line(r,r0,c0);
        return;

      case UP:
        store_line(r,r0,c0);
        if (r > r0)
          r--;
        break;

      case DN:
      case CR:
        store_line(r,r0,c0);
        if (r < r1)
          r++;
        else
          r = r0;
        break;
      case Ctrl_Y:
        store_line(r,r0,c0);
    }
  }
}

/*---------------------------------------------------------------------------*/
void
store_line(int r, int r0, int c0)
{
  int i = r-r0;
  int c,t;
  int ch;

  /* read string on row r */
  for (t=0,c=c0; t<ITEMSZ; t++,c++) {
    ch = read_char(c+dx, r+dy);
    if (ch == 0) ch = ' ';
    string[i][t] = ch;
  }
  string[i][ITEMSZ] = '\0';
}

/* ---------------------------------------------------------------------- */
/* put specified item on next year's calendar */

char string1[NUMITEMS][ITEMSZ+1];  /* global so it will be initialized to 0 */
carryover(int date, char *item)
{
  int month, day, nextyear, date1, leap1;
  int CALSZ1;
  char temp[10];
  FILE *gp1;
  struct ffblk ffblk;
  long file_size;

  int i,j;
  long zero = 0L;
  nextyear = year+1;
  leap1 = nextyear%4 == 0 && nextyear%100 != 0 || nextyear%400 == 0;
  if (leap1) CALSZ1 = 366; else CALSZ1 = 365;
  if (leap1 == leap)
    date1 = date;
  else {
    month_day(year, date, &month, &day, temp);
    date1 = day_of_year(month, day, nextyear);
  }
  if (findfirst(nextdata,&ffblk,0) != 0) {
    gp1 = fopen(nextdata,"wb");  /* if file doesn't exist, create it */
    for (i=0; i<CALSZ1; i++)
      fwrite(&zero, sizeof(long), 1, gp1);
    fclose(gp1);
  }
  gp1 = fopen(nextdata,"rb+");
  if (gp1 == NULL)
    normal_exit("Unable to find or create nextdata");
  if (fread(entered1, sizeof(long), CALSZ1, gp1) != CALSZ1) {
    normal_exit("Error reading 'nextdata' file");
    exit(1);
  }
  rewind(gp1);
  for (j=0; j<NUMITEMS; j++)  string1[j][0] = '\0';
  j = 0;
  if (entered1[date1]) {
    fseek(gp1, entered1[date], SEEK_SET);
    fread(string1, sizeof(char), sizeof(string1), gp1);
    for (j=0; j<NUMITEMS; j++)
      if (string1[j][0] != '\0')
        break;
    if (j == NUMITEMS) {    /* date1 already filled up for next year */
      fclose(gp1);
    return 0;
    }
  }
  else {
    fseek(gp1, 0L, SEEK_END);
    entered1[date1] = ftell(gp1);
  }
  strcpy(string1[j], item);
  fseek(gp1, entered1[date1], SEEK_SET);
  fwrite (string1, sizeof(char), sizeof(string1), gp1);
  fseek(gp1, 0L, SEEK_SET);
  j = fwrite(entered1, sizeof(long), CALSZ1, gp1);
  if (j != CALSZ1)
    normal_exit("Error writing 'entered[1]] to caldata.yy1");
  fclose(gp1);
  return 1;
}

/* ---------------------------------------------------------------------- */
/* gstr(s,col,row,len)
            General line entry and editing function. A replacement for
        cgets(), sort of, but with many editing enhancements. The line
        being edited is echoed in current screen background and text color.
        The header files conio.h, string.h and dos.h are required. The
        parameters col,row determine the cursor starting position. Parameter
        'len' is the maximum number of characters allowed, which should be
        within the capacity of s. Wraparound is not implemented, and the
        calling function should insure that 'col'+'len' does not exceed the
        right edge of the current text window, and that after omission of
        trailing spaces, the same is true for 'col' + strlen(s).
             The string s need not be empty at the start but trailing spaces
        will be eliminated. HOME and END move to the start and end positions,
        Ctrl_END clears from cursor to end of line, Ctrl_Y sets s to "" and
        returns and Ctrl_L restores the initial s (minus trailing spaces).
        The left and right arrow keys, Backspace, Ins and Del work as in DOS.
             Input is terminated by CR, Up, DN, Esc,or Ctrl_Y, and the return
        value is whichever one of these caused the exit. (The inclusion of UP
        and DN in this list is to allow gstr() to be used in a general text
        input program, with different strings to be edited on different rows
        of the input window.)
*/

gstr(char *s, int col, int row, int len)
{
   int i, c, pos=0, maxpos, end_s;
   char s0[80];
   int done = 0, insert_mode = 0;  /* start in overwrite mode */

   setcursor(insert_mode);
   maxpos = len;
   /* remove trailing spaces from s */
   i = strlen(s)-1;
   if (i > 0) {
     while (s[i] == ' ') i--;
     s[i+1] = '\0';
   }
   strcpy(s0,s);                   /* save initial string */
   end_s = strlen(s);
   pos = 0;
   gotoxy(col,row);
   cputs(s);
   gotoxy(col,row);

   while (1) {
     switch(c = getkey()) {
       case Ins:
         insert_mode = 1 - insert_mode;    /* toggle insert flag */
         setcursor(insert_mode);           /* adjust cursor to fit */
         break;
       case RT:
         if (pos < maxpos && pos < end_s)
           gotoxy(++pos+col,row);
         break;
       case LF:
         if (pos > 0)
           gotoxy(--pos+col,row);
         break;
       case Home:
         pos = 0;
         gotoxy(col+pos,row);
         break;
       case End:
         pos = end_s;
         gotoxy(col+pos,row);
         break;
       case Ctrl_END:
         clreol();
         s[pos] = '\0';
         end_s = strlen(s);
         break;
       case BK:                   /* backspace */
         if (pos > 0) {
         gotoxy(--pos+col, row); cputs(" ");
         gotoxy(pos+col, row);
         break;
       case Del:
         if (pos >= end_s)
           break;
         for (i=pos; i<end_s; i++)
           s[i] = s[i+1];          /* shift s left 1 char, starting at pos */
         end_s--;
         cputs(s+pos);
         cputs(" ");
         gotoxy(pos+col,row);
         break;
       case UP:
       case DN:
       case CR:
       case ESC:
         done = 1;
         break;
       case Ctrl_Y:
         s[0] = 0;
         gotoxy(col,row);
         clreol();
         done = 1;
         break;
       case Ctrl_L:
         strcpy(s,s0);
         gotoxy(col,row);
         clreol();
         cputs(s);
         pos = 0;
         end_s = strlen(s);
         break;

       default:
         if (!isprint(c) || pos == maxpos || pos > end_s)
           break;
         /* now have printable character c and cursor at some point in s */
         if (pos == end_s) {
           putch(c);             /* overwrite null in s */
           s[pos++] = c;
           s[pos] = 0;
           end_s++;
           gotoxy(pos+col,row);
           break;
         }
         if (insert_mode == 0) {
           putch(c);             /* overwrite current character in s */
           s[pos++] = c;
           gotoxy(pos+col,row);
           break;
         }
         if (end_s == maxpos)
           break;
         /* insert flag is set, there is room in s and on screen */
         end_s++;
         for (i=end_s; i>pos; i--)
           s[i] = s[i-1];        /* shift s right 1 char, starting with pos */
         s[pos] = c;             /* insert new character in s */
         cputs(s+pos++);         /* overprint part of s from pos on */
         gotoxy(pos+col,row);
         break;
       }
     }
     if (done)
       break;
   }
   setcursor(0);
   return (c);         /* returns UP, DN, ESC, or Ctrl_Y */
}

void
setcursor(int style)
{
  unsigned cx;

  if (style == 1) cx = 0x060C;         /* large, for insert mode */
  else if (style == 2) cx = 0x0200;    /* hide cursor */
  else cx = 0x0B0C;                    /* normal cursor is default */

  _AH = 1;
  _BH = 0;
  _CX = cx;
  geninterrupt(0x10);
}


/*     extract n integer values from string s, put them in array vi[]     */
Getint(int n, char *s, int vi[])
{
  int cnt=0;
  char *p;

  if (n == 0) return cnt;

  p = s;
  while (cnt < n && *p)
    {
     while (*p && !isdigit(*p)) p++;
     if (!*p)
       break;
     vi[cnt] = atoi(p);
     while (isdigit(*p)) p++;
     cnt++;
    }
  return cnt;
}

