/* CATALOG.C "Final" version */

#include "catalog.h"

/* global variables */
BOOK catalog[MAXSIZ];
int indx[MAXSIZ];          /* for ordering by title or author */
FILE *fp;                  /* catalog file */
int num_books;
int norm_attr = WHITE+16*BLUE, high_attr = BLUE+16*LIGHTGRAY;
int tag_norm = YELLOW+16*BLUE, tag_high = YELLOW+16*LIGHTGRAY;
int std_attr = BLUE+16*CYAN, error_attr = LIGHTGRAY+16*RED;
int current = 0, mono = 0;
int old_attr;
char filename[14];
int changed;
int exact = 0;

main(int argc, char *argv[])
{
  int choice,k=0;
  char s[32];
  int left,top,right,bottom;
  int col0, row0;
  int border;
  char *item[] =  {" Make new catalog  ",
                   " Load old catalog  ",
                   " Edit current data ",
                   " Save current data ",
                   " Quit ",
                     NULL };
  int ltr[] = {'m','l','e','s','q'};
  struct text_info oldtext;

  left = 30, top = 5; right = 50, bottom = 11;
  border = 2;
  col0 = 2, row0 = 3;

  if (argc==2) {
    if (strstr(strupr(argv[1]),"/E"))
      exact = 1;
    else
      strcpy(filename,argv[1]);
  }
  else if (argc > 2) {
    if (strstr(strupr(argv[1]),"/E")) {
      exact = 1;
      strcpy(filename,argv[2]);
    }
    else if (strstr(strupr(argv[2]),"/E")) {
      exact = 1;
      strcpy(filename,argv[1]);
    }
    else {
      exact = 0;
      strcpy(filename,argv[1]);
    }
  }
  gettextinfo(&oldtext);           /* original text settings */
  old_attr = oldtext.normattr;
  if (oldtext.currmode == '\7') {
    mono = 1;
    norm_attr = tag_norm = std_attr = 7;
    high_attr = tag_high = error_attr = 16*7;
  }
  if (filename[0]) {
    stdscrn();
    box(27,10,53,12,high_attr,1);
    window(27,10,53,12);
    gotoxy(2,1); showcursor();
    cprintf("filename: %s",filename);
    if (exact)
      k = read_lines_E(filename);
    else
      k = read_lines(filename);
    changed = 0;    /* read_lines() calls insert, which sets changed = 1 */
  }
  if (k > 0)        /* no error reading file */
    edit();
  while(1) {
    stdscrn();
    gotoxy(32,25);
    cputs("Press F1 for help");
    window(1,1,80,24);
    switch(menu_select(left, top, right, bottom, border," MAIN ",
                      col0, row0, current, ltr, item)) {
        case -1:
        case 4:
          normal_exit();
        case -2:
          help(1);
          break;
        case 0:                 /* make new catalog */
          create();
          if (num_books == 0)
            current = 0;
          else
            current = 2;        /* after creating catalog, edit it? */
          break;
        case 1:
a:
          box(27,14,53,14,high_attr,1);
          gotoxy(2,2); showcursor();
          cputs("filename: ");
          filename[0] = 0;
          if (Getstr(filename,13) == -1)
            break;
          if (!filename[0])
            break;
          strupr(filename);
          if (exact)
            k = read_lines_E(filename);
          else
            k = read_lines(filename);
          changed = 0;
          if (k == -1) {
            textattr(std_attr);
            window(1,16,80,20);
            clrscr();
            goto a;
          }
          if (k == 0) {    /* catalog empty */
            error_msg("No data found in file",27,16);
            current = 1;
            break;
          }
          current = 2;         /* after loading catalog, edit it? */
          changed = 0;         /* insert() is called by read_lines() */
          break;
      case 2:
        edit();
        break;
      case 3:
        save();
    }
  }
}

/* ---------------------------------------------------------------------- */
void
create()          /* make new catalog file */
{
  int i;

  num_books = 0;
  for (i=0; i<MAXSIZ; i++) indx[i] = i;
  stdscrn();
  gotoxy(2,22);
  cputs("Maximum lengths:");
  gotoxy(5,23);
  cputs("title = 50    author = 30   id code = 20");
  showcursor();
  while (enter_book())
    ;
}

/* ---------------------------------------------------------------------- */
void
edit()
{
  char *item[] =  {" Add books      ",
                   " List entries   ",
                   " Search catalog ",
                   " Modify entry   ",
                     NULL};
  int ltr[] = {'a','l','s','m'};
  int left = 30, top = 5, right = 50, bottom = 8;
  int border = 2;
  int col0 = 2, row0 = 2, choice;
  while(1) {
    stdscrn();
    gotoxy(32,25);
    cputs("Press F1 for help");
    window(1,1,80,24);
    switch(menu_select(left, top, right, bottom, border," EDIT ",
                   col0, row0, 0, ltr, item)) {
      case -1:
        return;
      case -2:
        help(2);
        break;
      case 0:
        if (num_books == MAXSIZ - 1) {
          error_msg("Catalog full",left+1,top+1);
          break;
        }
	stdscrn();
	gotoxy(2,22);
	cputs("Maximum lengths:");
	gotoxy(5,23);
	cputs("title = 50    author = 30   id code = 20");
	showcursor();
        while (enter_book())
          ;
        break;
      case 1:
        list();
        break;
      case 2:
        search();
        break;
      case 3:
        modify();
        break;
    }
  }
}

/* ---------------------------------------------------------------------- */
void
list()
{
  char *item[] =  {" Authors ",
                   " Titles  ",
                   " Catalog ",
                     NULL};
  int ltr[] = {'a','t','c'};
  int left=33,top=5,right=48,bottom=10;
  int border=2,choice;
  int col0 = 4,row0 = 3;
  while(1) {
    stdscrn();
    gotoxy(32,25);
    cputs("Press F1 for help");
    window(1,1,80,24);
    choice = (menu_select(left,top,right,bottom,border," LIST ",
		      col0, row0, 0, ltr, item));
    if (choice == -1)
        return;
    if (choice == -2)
        help(3);
    make_list(choice);     /* display or print authors, titles or catalog */
  }
}

/* ---------------------------------------------------------------------- */
void
save()
{
  FILE *fp;
  int i;
  char savename[14];

  if (num_books == 0) {
    error_msg("No entries in catalog",29,10);
    return;
  }
  box(27,14,53,14,high_attr,1);
  gotoxy(2,2); showcursor();
  savename[0] = 0;
  cputs("filename: ");
  if (Getstr(savename,13) == -1)
    return;
  if (!savename[0])
    if (filename[0])
      strcpy(savename,filename);
    else
      return;
  fp = fopen(savename,"w");
  for (i=0; i<num_books; i++) {
    fprintf(fp,"%s\n",catalog[indx[i]].author);
    fprintf(fp,"%s\n",catalog[indx[i]].title);
    fprintf(fp,"%s\n",catalog[indx[i]].ident);
  }
  fclose(fp);
  changed = 0;
}

/* ---------------------------- exit ------------------------------------ */
void
normal_exit()
{
  window (1,1,80,25);
  textattr(old_attr);
  showcursor();
  clrscr();
  if (changed) {
    cprintf("Save changed file (Y/N)? ");
    if (toupper(getch()) == 'Y') {
      save();
    }
    window(1,1,80,25);
    textattr(old_attr);
    clrscr();
  }
  exit(0);
}

