/* CAT_MENU.C Module for catalog1 containing the menu and box functions */

#include "catalog.h"
#include "catvars.h"


/* ------------------------- show menu, make selection -------------------- */
menu_select(scol,srow,ecol,erow,        /* box sides */
            border,                     /*  0 = none, 1 = single, 2 = double */
            title,                      /* centered on top border */
            col0, row0,                 /* start column,row in window */
            current,                    /* currently highlighted item */
            ltr,                        /* highlighted letters */
            item)                       /* list of menu items; last = NULL */
int scol,srow,ecol,erow,border,col0,row0,current,*ltr;
char *title, *item[];
{
  int i, c, max;
  int col;

  hidecursor();
  textattr(std_attr);
  clrscr();
  box(scol,srow,ecol,erow,norm_attr,border);
  if (title[0] && strlen(title) < ecol-scol-2) {
    col = (scol+ecol-strlen(title))/2 - scol + 2;
    gotoxy(col,1);
    cputs(title);
  }
  if (border == 0)
    row0--;
  i = 0;
  while (item[i] != NULL) {
    norm(1+col0,i+row0,item[i]);
    i++;
  }
  max = i-1;
  high(1+col0,current+row0,item[current]);

  while (kbhit())
    getch();                              /* clear kb buffer */
  while ((c=getkey()) != 27)              /* until Esc key pressed */
    {
     switch(c)
      {
       case -72:        /* Up arrow */
         norm(1+col0,current+row0,item[current]);
         if (current == 0)
           current = max;
         else
           current--;
         high(1+col0,current+row0,item[current]);
         break;
       case -80:        /* Down arrow */
         norm(1+col0,current+row0,item[current]);
         if (current == max)
           current = 0;
         else
           current++;
         high(1+col0,current+row0,item[current]);
         break;
        case 13:                   /* make selection when Enter key pressed */
          return current;
        case F1:
          return -2;
        default:      /* if letter command, change highlight and select */
          for (i=0; i<=max; i++)
            if (ltr[i] == c || ltr[i]-32 == c)
              {
               norm(1+col0,current+row0,item[current]);
               current = i;
               high(1+col0,current+row0,item[current]);
               return current;
              }
      }
    }
    return -1;                     /* get here only if Esc key pressed */
}

/* -------------------------------------- */
void
norm(int col, int row, char *s)
{
  textattr(norm_attr);
  gotoxy(col,row);
  cputs(s);
  textattr(tag_norm);
  gotoxy(col+1,row);
  cprintf("%c",s[1]);
}

/* -------------------------------------- */
void
high(int col,int row, char *s)
{
  textattr(high_attr);
  gotoxy(col,row);
  cputs(s);
  textattr(tag_high);
  gotoxy(col+1,row);
  cprintf("%c",s[1]);
}


/* --------------------------------------------------------------------- */
/* filled box, attribute = attr */
/* no border if border = 0, single if border = 1, double if border = 2 */
int tlc[3]={0,218,201};
int trc[3]={0,191,187};
int blc[3]={0,192,200};
int brc[3]={0,217,188};  /* single,double corners */
int hl[3]={0,196,205};
int vl[3]={0,179,186};   /* single,double horizontal, vertical lines */

void
box(int scol, int srow, int ecol, int erow, int attr, int border)
{
  int i;
  int width,height;
  int row;

  textattr(attr);
  row = srow;
  width = ecol-scol+1;
  height = erow-srow+1;
  if (border) {
    srow--; erow++; scol--; ecol++;
  }
  if (scol<1 || srow<1 || ecol>80 || erow>25) {
    window(1,1,80,25);
    textattr(7);
    clrscr();
    printf(
    "Error in box coordinates: scol = %d, srow = %d, ecol = %d, erow = %d\n",
    scol,srow,ecol,erow);
    exit(1);
  }
  window(scol,srow,ecol,erow);
  clrscr();
  if (border) {
    window(1,1,80,25);
    gotoxy(scol,srow);
    putch(tlc[border]);
    for (i=0; i<width; i++)
      putch(hl[border]);
    putch(trc[border]);
    for (i=0; i<height; i++) {
      gotoxy(scol,row);
      putch(vl[border]);
      gotoxy(ecol,row++);
      putch(vl[border]);
    }
    gotoxy(scol,erow);
    putch(blc[border]);
    for (i=0; i<width; i++)
      putch(hl[border]);
    putch(brc[border]);
    window(scol,srow,ecol,erow);
  }
}

