/* CAT_UTIL.C  Various utilities for CATALOG */

#include "catalog.h"
#include "catvars.h"

/* ------------------------- normalize name --------------------------- */
clean_name(char *s)
{
  char t[ASIZ+10], *p, *q;  /* leave space in t[] for possible added chars */
  int i, first = 1, apostrophe = 39, hyphen = 45, comma = 44;
  int cnt;

  s[ASIZ-1] = '\0';         /* if too long, chop off */
  for (i=0; i<ASIZ; i++) t[i] = '\0';
  clean_spaces(s,ASIZ);
  p=s, q=t;
  while (1) {
    while (*p && !isalpha(*p)) p++;
    if (*p == 0)               /* nothing but non-alphas */
      return 0;
    if (capitalize_name(p) != 0)
      *q++ = toupper(*p++);
    else
      *q++ = *p++;
    if (*p == '\0')
      break;
    cnt = 1;
    while (*p && (isalpha(*p) || *p == apostrophe || *p == hyphen)) {
      if (*(p-1) == apostrophe || *(p-1) == hyphen)
        *q++ = toupper(*p++);
      else
        *q++ = *p++;
      cnt++;
    }
    if (first) {
      if (cnt == 1)
        return 0;       /* do not accept last name with only one letter */
    }
    if (first) {
      while (*p && *p == ' ')
        p++;
      if (*p == '\0')
        break;
      if (!isalpha(*p) && *p != comma)
        return 0;
      }
    if (first && *p == comma) {
      *q++ = *p++;
      first = 0;
    }
    while (*p && !isalpha(*p)) p++;
    if (*p == 0)
      break;
    if (cnt == 1)
      *q++ = '.';       /* assume single letters are initials */
    *q++ = ' ';
  }
  if (cnt == 1 || *(q-2) == ' ')
    *q++ = '.';
  *q = '\0';
  strcpy(s,t);
  s[ASIZ-1] = '\0';     /* to be sure modified string is not too large */
  return 1;
}

/* ---------------------------------------------------------------------- */
clean_title(char *s)
{
  char t[TSIZ], *p, *q;
  int i,L;

  s[TSIZ-1] = '\0';         /* if too long, chop off */
  /* remove unneeded spaces, capitalize, add periods, etc. */
  for (i=0; i<TSIZ; i++) t[i] = '\0';
  p=s, q=t;
  while (1) {
    while (*p && (!isalnum(*p) && !ispunct(*p)) || isspace(*p)) p++;
    if (*p == 0)               /* nothing acceptable */
      return 0;
    if (p == s || capitalize_title(p))
      *q++ = toupper(*p++);
    else
      *q++ = tolower(*p++);
    while (*p && (isalnum(*p) || ispunct(*p))) {
      *q++ = tolower(*p++);
    }
    while (*p && !isalnum(*p) && !ispunct(*p)) p++;
    if (*p == 0)
      break;
    if (!ispunct(*p))
      *q++ = ' ';
  }
  *q = '\0';
  if (t[0] == 'T' && t[1] == 'h' && t[2] == 'e' && t[3] == ' ') {
    L = strlen(s);
    for (i=4; i<=L; i++) t[i-4] = t[i];
    L = strlen(t);
    if (L < TSIZ-5)
      strcat(t,", The");
    else if (L == TSIZ-5)
      strcat(t,",The");
  }
  t[TSIZ-1] = '\0';
  strcpy(s,t);
  return 1;
}

/* ------------------------------------ */
capitalize_title(char *p)
{
  if (strncmpi(p,"a ",2) == 0)
    return 0;
  if (strncmpi(p,"an ",3) == 0)
    return 0;
  if (strncmpi(p,"and ",4) == 0)
    return 0;
  if (strncmpi(p,"be ",3) == 0)
    return 0;
  if (strncmpi(p,"by ",3) == 0)
    return 0;
  if (strncmpi(p,"if ",3) == 0)
    return 0;
  if (strncmpi(p,"in ",3) == 0)
    return 0;
  if (strncmpi(p,"is ",3) == 0)
    return 0;
  if (strncmpi(p,"it ",3) == 0)
    return 0;
  if (strncmpi(p,"of ",3) == 0)
    return 0;
  if (strncmpi(p,"on ",3) == 0)
    return 0;
  if (strncmpi(p,"for ",4) == 0)
    return 0;
  if (strncmpi(p,"from ",5) == 0)
    return 0;
  if (strncmpi(p,"the ",4) == 0)
    return 0;
  return 1;
}

/* ------------------------------------ */
capitalize_name(char *p)
{
  if (strncmpi(p,"von ",4) == 0)
    return 0;
  if (strncmpi(p,"van ",4) == 0)
    return 0;
  if (strncmpi(p,"and ",4) == 0)
    return 0;
  if (strncmpi(p,"de ",3) == 0)
    return 0;
  if (strncmpi(p,"di ",3) == 0)
    return 0;
  if (strncmpi(p,"d'",2) == 0)
    return 0;
  return 1;
}

/* ------------------------------- */
clean_spaces(char *s, int n)
{
  char *p, *q;
  char t[ASIZ];

  p = s, q = t;
  while (1) {
    while (*p && !isprint(*p) || isspace(*p)) p++;
    if (*p == 0)               /* nothing acceptable */
      return 0;
    *q++ = *p++;
    while (*p && isprint(*p) && !isspace(*p))
      *q++ = *p++;
    if (*p == '\0')
      break;
    while (isspace(*p)) p++;
    if (*p == 0)
      break;
    if (!ispunct(*p))
      *q++ = ' ';
  }
  *q = '\0';
  t[n] = '\0';
  strcpy(s,t);
  return 1;
}

/* ---------------------------------------------------------------------- */
error_msg(char *msg, int col, int row)  /* col,row absolute coordinates */
{
  char buf[160], msg1[80];
  int length,c;
  struct text_info text;

  gettextinfo(&text);
  window(1,1,80,25);
  length = strlen(msg) + 2;
  if (col+length > 80)           /* not enough room for message */
    return 0;
  msg1[0] = ' ';                 /* add spaces at start and end of msg */
  strcpy (&msg1[1],msg);
  strcat(msg1," ");
  gettext(col, row, col+length, row, buf);   /* save current screen area */
  textattr(error_attr);
  gotoxy(col,row);
  cputs(msg1);
  gotoxy(wherex()-1,row);         /* in case cursor is on */
  c = getch();
  puttext(col,row,col+length,row,buf);
  textattr(text.normattr);
  window(text.winleft,text.wintop,text.winright,text.winbottom);
  return c;
}

/* ------------------------- general input -------------------------------- */
getkey()
{
  int k;

  k = getch();
  if (k == 0)
    k = -getch();
  return k;
}

/* --------------- special search function for s in t ------------------ */
/* --------------- converts t to uppercase before searching ------------ */
instr(char *t, char *s)
{
  char temp[TSIZ];

  strcpy(temp,t);
  strupr(temp);
  if (strstr(temp,s) == NULL)
    return 0;
  else
    return 1;
}

/* ------------------------- hide the cursor -------------------------- */
void
hidecursor()
{
  _AH = 1;
  _CH = 0x20;
  geninterrupt(0x10);
}

/* ------------------------- show the cursor ------------------------------ */
void
showcursor()
{
  _AX = 0x0100;
  _BX = 0;
  _CX = 0x0c0d;
  geninterrupt(0x10);
}

/* ------------------------ scroll window n lines ----------------------- */
void
scroll(int left, int top, int right, int bottom, int attr, int n, int dir)
{
  if (dir == 1)
    _AH = 6;
  else
    _AH = 7;
  _AL = n;              /* scroll one line */
  _BH = attr;
  _CH = top-1;
  _CL = left-1;
  _DH = bottom-1;
  _DL = right-1;
  geninterrupt(0x10);
}

/* ----------------------- standard screen for menus ------------------- */
void
stdscrn()
{
  window(1,1,80,25);
  textattr(std_attr);
  clrscr();
}

