/* GETSTR.C  General string input function */
#include <conio.h>
#include <string.h>
#include <dos.h>
#include <ctype.h>

void
close_up(int c, int c1, int r);
void
read_str(char *s, int c0, int c1, int r);

/* ------------------------- string input -------------------------------- */
/* Getstr(s,len) Replacement for cgets(). This function displays input in
               current screen background and text color. As with cgets(),
               the cursor must be set to the input position in advance.
               The string 's' need not be empty at the outset, but must
               be less than 'len' characters in length. It must be short
               enough to fit on the input line. Input is terminated by CR
               or by Esc. Returns -1 if Esc pressed, 1 otherwise.

               Getstr(s,len) willnot accept more than 'len' characters
               and will not read beyond the end of the current line.
*/
/*---------------------------------------------------------------------------*/

Getstr(char *s, int len)
{
  int i,c,r,c0,c1;
  int k,done=0, end_s;
  int insert_mode = 0;
  char ch;

  c=c0=wherex();
  r = wherey();
  if (80-c0 < len) len = 80-c0;
  c1 = c0+len;
  if (strlen(s) > len) s[len] = '\0';
  gotoxy(c,r);
  cputs(s);
  c = end_s = c0+strlen(s);
  while(1) {
    gotoxy(c,r);
    k = getch();
    if (k == 27)
      return -1;
    if (k==0) k = -getch();
    switch(k) {
      case -77:        /* right arrow */
        if (c < c1)
          c++;
          if (c > end_s) end_s = c;
        break;
      case -75:        /* left arrow */
        if (c > c0)
          c--;
        break;
      case 8:          /* backspace */
        if (c == c0)
          break;
        if (c == end_s) end_s--;
        c--;
        gotoxy(c,r);
        cputs(" ");
        break;
      case -83:        /* Del key */
        cputs(" ");
        close_up(c,c1,r);
        end_s--;
        break;
       case -82:       /* Ins key */
         insert_mode = 1 - insert_mode;    /* toggle insert flag */
         setcursor(insert_mode);           /* adjust cursor to fit */
         break;
       case -71:       /* Home key */
         c = c0;
         break;
       case -79:       /* End key */
         c = end_s;
         break;
      case 13:
        done = 1;
        break;
      default:
        if (isprint(k) && c<c1) {
          if (insert_mode == 0) {
            putch(k);    /* overwrite current character, if any */
            c++;
            if (c > end_s) end_s = c;
          }
          else {  /* insert is set and there is room for another character */
            setcursor(2);           /* turn off cursor */
            for (i=c1; i>c; i--) {  /* shift tail of string to right one char */
              ch = read_char(i-1,r);
              gotoxy(i,r);
              putch(ch);
            }
            gotoxy(c,r);             /* back to original position */
            putch(k);                /* replace current character with new one */
            c++;                     /* increment position */
            setcursor(insert_mode);  /* show cursor again */
          }
       }
    }
    if (done) {
      setcursor(0);                  /* normal cursor */
      read_str(s,c0,end_s,r);        /* read characters on screen into s */
      gotoxy(end_s,r);
      return 1;
    }
  }
}

/*----------- part of gstr() --------------------*/
/* shift tail of s left one char */
void
close_up(int c, int c1, int r)
{
  int i;
  char ch;

  for (i=c; i<c1-1; i++) {
    ch = read_char(i+1,r);
    gotoxy(i,r);
    putch(ch);
  }
  gotoxy(c1-1,r);
  putch(' ');
}

/*----------- part of gstr() --------------------*/
/* read character on screen at (c,r) */
int
read_char(int c, int r) 
{
  gotoxy(c,r);
  _AH = 8;
  _BH = 0;
  geninterrupt(0x10);
  return _AL;
}

/*----------- part of gstr() --------------------*/
/* read chars from (c0,r)-(c1,r) into s */
void
read_str(char *s, int c0, int c1, int r)
{
  int c;
  char *p = s;

  for (c=c0; c<c1; c++) {
    *p = read_char(c,r);
    p++;
  }
  *p = '\0';
  while (*(--p) == ' ')
    ;
  *(p+1) = '\0';
}

/* ------------------------ change cursor size -------------------------- */
setcursor(int style)
{
  unsigned cx;

  if (style == 1) cx = 0x060C;         /* large, for insert mode */
  else if (style == 2) cx = 0x0200;    /* hide cursor */
  else cx = 0x0B0C;                    /* normal cursor is default */

  _AH = 1;
  _BH = 0;
  _CX = cx;
  geninterrupt(0x10);
}

